"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.IsValidIUserConnectionWithUser = exports.IsBasicAuthPasswordValid = exports.AreOAuthCredentialsValid = exports.IsConnectionTypeBasic = exports.IsConnectionTypeOAuth = exports.IsValidEnvironmentConnection = exports.SendOAuthAuthenticationRequestToScheduler = exports.ExtractRoleFromOAuthResponseData = exports.AddUserConnectionAndAccountToDeployRunInfo = exports.GetUserConnectionInfoByEnvironment = exports.IsValidIUserConnection = exports.SetUserConnectionDataByEnvironment = exports.CreateOAuthUserConnection = exports.CreateBasicUserConnection = exports.GetValueFromConnectionDetailsSafe = exports.DeleteUserConnectionSecretUID = exports.GetCurrentUserSecretFromFirebase = exports.SetAndRetrieveNewUserSecretFirebase = exports.GetEnvironmentAccountAndUserConnection = exports.GetOAuthDetailsFromEnvironment = exports.GetAccountFromEnvironment = exports.cleanConnectionDetailForLogging = exports.EFirebaseSecretCollectionAndDocNames = exports.EUserConnectionTypes = exports.isValidUserConnectionConfiguration = exports.EUserConnectionDetail = exports.FirebaseLogger = void 0;
const CommonOperations = __importStar(require("./CommonOperations"));
const ConnectionOperations = __importStar(require("./ConnectionOperations"));
const immer = __importStar(require("immer"));
const Workspaces = __importStar(require("./Workspaces"));
const Firebase = __importStar(require("./Firebase"));
const RunnerOperations = __importStar(require("./RunnerOperations"));
const Common = __importStar(require("./Common"));
const Logging = __importStar(require("./Logging"));
const Templates = __importStar(require("./Templates"));
const R = __importStar(require("runtypes"));
/**
 * TABLE OF CONTENTS
 * - 1 : SETUP
 * - 2 : ENUMS/INTERFACES
 * - 3 : FIREBASE
 * - 4 : CONNECTIONS
 * - 5 : OAUTH
 */
/**
 * ===== SETUP =====
 */
exports.FirebaseLogger = Logging.GetLogger(Logging.LoggingArea.Firebase);
const ConnLog = Logging.GetLogger(Logging.LoggingArea.Connection);
var EUserConnectionDetail;
(function (EUserConnectionDetail) {
    EUserConnectionDetail["userID"] = "userID";
    EUserConnectionDetail["user"] = "user";
    EUserConnectionDetail["role"] = "role";
    EUserConnectionDetail["warehouse"] = "warehouse";
    EUserConnectionDetail["keyPair"] = "keyPair";
})(EUserConnectionDetail = exports.EUserConnectionDetail || (exports.EUserConnectionDetail = {}));
const isValidUserConnectionConfiguration = (connection, newRunInfo) => {
    if (!connection.connectionType ||
        (connection.connectionType === ConnectionOperations.EUserConnectionTypes.basic && !newRunInfo.userCredentials.snowflakePassword) ||
        (connection.connectionType === ConnectionOperations.EUserConnectionTypes.keyPair && !connection.connectionDetails.keyPair)) {
        return false;
    }
    return true;
};
exports.isValidUserConnectionConfiguration = isValidUserConnectionConfiguration;
var EUserConnectionTypes;
(function (EUserConnectionTypes) {
    EUserConnectionTypes["basic"] = "Basic";
    EUserConnectionTypes["oauth"] = "OAuth";
    EUserConnectionTypes["keyPair"] = "KeyPair";
})(EUserConnectionTypes = exports.EUserConnectionTypes || (exports.EUserConnectionTypes = {}));
var EFirebaseSecretCollectionAndDocNames;
(function (EFirebaseSecretCollectionAndDocNames) {
    EFirebaseSecretCollectionAndDocNames["userSecrets"] = "userSecrets";
    EFirebaseSecretCollectionAndDocNames["currentSecretDoc"] = "secret";
    EFirebaseSecretCollectionAndDocNames["currentSecretIDProp"] = "currentSecretID";
})(EFirebaseSecretCollectionAndDocNames = exports.EFirebaseSecretCollectionAndDocNames || (exports.EFirebaseSecretCollectionAndDocNames = {}));
const RUserConnectionTypes = Common.RuntypeFromEnum(EUserConnectionTypes);
const RUserConnectionDetail = R.Record({
    userID: R.String,
    user: R.String,
    role: R.String,
    warehouse: R.String,
    authenticator: R.Optional(R.Union(//this is optional because its unused. there was a bug at some point that added this field 
    //so this is part of the definition - but its optional and should not be used. 
    R.Literal(ConnectionOperations.EUserConnectionTypes.basic), R.Literal(ConnectionOperations.EUserConnectionTypes.oauth), R.Literal(ConnectionOperations.EUserConnectionTypes.keyPair))),
    keyPair: R.Optional(R.Unknown) //Buffer
});
const RUserConnectionType = R.Record({
    connectionType: RUserConnectionTypes,
    connectionDetails: RUserConnectionDetail
});
const cleanConnectionDetailForLogging = (connectionDetail) => {
    return {
        userID: connectionDetail.userID,
        user: connectionDetail.user,
        role: connectionDetail.role,
        warehouse: connectionDetail.warehouse,
        authenticator: connectionDetail.authenticator
    };
};
exports.cleanConnectionDetailForLogging = cleanConnectionDetailForLogging;
/**
 * ===== FIREBASE =====
 */
const GetAccountFromEnvironment = (firestore, teamID, environmentID) => {
    return new Promise((resolve, reject) => {
        return Workspaces.getWorkspacesCollectionRefAdmin(firestore, teamID).doc(environmentID.toString())
            .get()
            .then((doc) => {
            const data = doc.data();
            const connectionAccount = (data === null || data === void 0 ? void 0 : data.connectionAccount) || "";
            resolve(connectionAccount);
        })
            .catch(reject);
    });
};
exports.GetAccountFromEnvironment = GetAccountFromEnvironment;
const GetOAuthDetailsFromEnvironment = (firestore, teamInfo, environmentID) => {
    return new Promise((resolve, reject) => {
        return Workspaces.getWorkspacesCollectionRefAdmin(firestore, teamInfo.fbTeamID).doc(environmentID.toString())
            .get()
            .then((doc) => {
            if (doc.exists) {
                const data = doc.data();
                const oauthDetails = {
                    oauthClientID: data === null || data === void 0 ? void 0 : data.oauthClientID,
                    oauthClientSecret: data === null || data === void 0 ? void 0 : data.oauthClientSecret,
                    connectionAccount: data === null || data === void 0 ? void 0 : data.connectionAccount
                };
                resolve(oauthDetails);
            }
            else {
                //doc doesn't exist, throw error
                throw new Error("Environment doesn't exist");
            }
        })
            .catch(reject);
    });
};
exports.GetOAuthDetailsFromEnvironment = GetOAuthDetailsFromEnvironment;
const GetEnvironmentAccountAndUserConnection = (firestore, teamInfo, environmentID) => {
    return new Promise((resolve, reject) => {
        const { fbTeamID: teamID } = teamInfo;
        return Promise.all([(0, exports.GetAccountFromEnvironment)(firestore, teamID, environmentID), (0, exports.GetUserConnectionInfoByEnvironment)(firestore, teamInfo, environmentID)])
            .then(connectionData => {
            const connectionReturnObject = {
                userConnection: connectionData[1],
                accountName: connectionData[0]
            };
            resolve(connectionReturnObject);
        })
            .catch(reject);
    });
};
exports.GetEnvironmentAccountAndUserConnection = GetEnvironmentAccountAndUserConnection;
const GetUserConnectionRefByEnvironment = (firestore, teamInfo, environmentID) => {
    return CommonOperations.getDocumentRefAdmin(firestore, teamInfo.fbTeamID, environmentID, CommonOperations.ECollectionRefs.users).doc(teamInfo.fbUserID);
};
/**
 * Generates a generic firebase doc to use its id as a UID for workspaces -> users -> userSecrets -> userDoc.currentUID
 * @param tokenData contains TeamOperations.teamInfo and a firebase instance to get documents from firestore
 * @param environmentID also known as the 'workspaceID'
 * @returns the id from the generated firebase doc
 */
const SetAndRetrieveNewUserSecretFirebase = (tokenData, environmentID) => {
    const { fbTeamID, fbUserID } = tokenData.teamInfo;
    return new Promise((resolve, reject) => {
        let outputID;
        const SecretCollection = GetUserConnectionRefByEnvironment(tokenData.firebase.firestore(), tokenData.teamInfo, environmentID).collection(EFirebaseSecretCollectionAndDocNames.userSecrets);
        const userSecretDoc = SecretCollection.doc(EFirebaseSecretCollectionAndDocNames.currentSecretDoc);
        const SecretGenerationDoc = SecretCollection.doc();
        SecretGenerationDoc.get()
            .then((data) => {
            outputID = data.id;
            return userSecretDoc.set({
                currentSecretID: data.id
            });
        })
            .then(() => {
            resolve(outputID);
        })
            .catch((err) => {
            exports.FirebaseLogger.errorContext({ orgID: fbTeamID, userID: fbUserID, envID: environmentID }, "SetAndRetrieveNewUserSecretFirebase", err);
            console.log("Error obtaining secretID", err);
            reject(err);
        });
    });
};
exports.SetAndRetrieveNewUserSecretFirebase = SetAndRetrieveNewUserSecretFirebase;
/**
 * retrieves the UID value from a workspaces -> users doc
 * @param tokenData contains TeamOperations.teamInfo and a firebase instance to get documents from firestore
 * @param environmentID also known as the 'workspaceID'
 * @returns the current UID in a promise
 */
const GetCurrentUserSecretFromFirebase = (tokenData, environmentID) => {
    const { teamInfo, firebase } = tokenData;
    const { fbTeamID, fbUserID } = teamInfo;
    return new Promise((resolve, reject) => {
        const SecretCollection = GetUserConnectionRefByEnvironment(firebase.firestore(), teamInfo, environmentID).collection(EFirebaseSecretCollectionAndDocNames.userSecrets);
        const UserSecretDoc = SecretCollection.doc(EFirebaseSecretCollectionAndDocNames.currentSecretDoc);
        UserSecretDoc.get()
            .then((doc) => {
            resolve(doc.data().currentSecretID);
        })
            .catch((err) => {
            exports.FirebaseLogger.errorContext({ orgID: fbTeamID, userID: fbUserID, envID: environmentID }, "GetCurrentUserSecretFromFirebase", err);
            reject(err);
        });
    });
};
exports.GetCurrentUserSecretFromFirebase = GetCurrentUserSecretFromFirebase;
const DeleteUserConnectionSecretUID = (tokenData, environmentID) => {
    return new Promise((resolve, reject) => {
        return GetUserConnectionRefByEnvironment(tokenData.firebase.firestore(), tokenData.teamInfo, environmentID).collection(EFirebaseSecretCollectionAndDocNames.userSecrets).doc(EFirebaseSecretCollectionAndDocNames.currentSecretDoc).delete()
            .then(() => resolve())
            .catch(reject);
    });
};
exports.DeleteUserConnectionSecretUID = DeleteUserConnectionSecretUID;
/**
 * ===== CONNECTIONS =====
 */
const GetValueFromConnectionDetailsSafe = (userConnection, param) => {
    if (!userConnection) {
        return "";
    }
    return Common.getValueSafe(userConnection, ["connectionDetails", param], "");
};
exports.GetValueFromConnectionDetailsSafe = GetValueFromConnectionDetailsSafe;
const CreateBasicUserConnection = (userID) => {
    return {
        connectionType: EUserConnectionTypes.basic,
        connectionDetails: {
            user: "",
            role: "",
            userID: userID,
            warehouse: "",
        }
    };
};
exports.CreateBasicUserConnection = CreateBasicUserConnection;
const CreateOAuthUserConnection = (userID, user, role) => {
    return {
        connectionType: EUserConnectionTypes.oauth,
        connectionDetails: {
            userID,
            user,
            role: role,
            warehouse: "",
        }
    };
};
exports.CreateOAuthUserConnection = CreateOAuthUserConnection;
const SetUserConnectionDataByEnvironment = (firestore, teamInfo, environmentID, connectionData) => {
    return new Promise((resolve, reject) => {
        return GetUserConnectionRefByEnvironment(firestore, teamInfo, environmentID)
            .set(connectionData, { merge: true })
            .then(() => {
            resolve();
        })
            .catch(reject);
    });
};
exports.SetUserConnectionDataByEnvironment = SetUserConnectionDataByEnvironment;
const IsValidIUserConnection = (connectionData) => {
    const queryValidationResult = Templates.validateRunType(RUserConnectionType, connectionData);
    return queryValidationResult.success;
};
exports.IsValidIUserConnection = IsValidIUserConnection;
const GetUserConnectionInfoByEnvironment = (firestore, teamInfo, environmentID) => {
    const logContext = {
        orgID: teamInfo.fbTeamID,
        userID: teamInfo.fbUserID,
        envID: environmentID
    };
    return new Promise((resolve, reject) => {
        return GetUserConnectionRefByEnvironment(firestore, teamInfo, environmentID)
            .get()
            .then(doc => {
            if (doc.exists && (0, exports.IsValidIUserConnection)(doc.data())) {
                const data = doc.data();
                const connection = {
                    connectionType: data.connectionType,
                    connectionDetails: {
                        user: data.connectionDetails.user,
                        userID: data.connectionDetails.userID,
                        role: data.connectionDetails.role,
                        warehouse: data.connectionDetails.warehouse,
                    }
                };
                ConnLog.infoContext(logContext, "User Connection found, type:", connection.connectionType);
                resolve(connection);
            }
            else {
                throw new Error(`Connection for user:${teamInfo.fbUserID} of org:${teamInfo.fbTeamID} in env:${environmentID} does not exist or is not valid. Please check the connection setup`);
            }
        })
            .catch(error => {
            ConnLog.errorContext(logContext, "Error getting user connection info", error);
            reject(error);
        });
    });
};
exports.GetUserConnectionInfoByEnvironment = GetUserConnectionInfoByEnvironment;
const AddUserConnectionAndAccountToDeployRunInfo = (userConnection, runInfo, userID) => {
    const { toWorkspaceID } = runInfo;
    return immer.produce(runInfo, (draftState) => {
        draftState.fromWorkspaceData.userConnections = { [toWorkspaceID]: { [userID]: userConnection } }; //what if userID is undefined?
    });
};
exports.AddUserConnectionAndAccountToDeployRunInfo = AddUserConnectionAndAccountToDeployRunInfo;
const ExtractRoleFromOAuthResponseData = (oauthData) => {
    const scope = oauthData.scope;
    const role = scope.split("role:")[1];
    return role;
};
exports.ExtractRoleFromOAuthResponseData = ExtractRoleFromOAuthResponseData;
/**
 * ===== OAUTH =====
 */
const SendOAuthAuthenticationRequestToScheduler = (token, workspaceID, oauthClientID, oauthClientSecret, authURL, tokenURL, redirect_uri) => {
    var _a;
    const fbToken = (_a = Firebase.getFirebaseAuth().currentUser) === null || _a === void 0 ? void 0 : _a.refreshToken;
    return RunnerOperations.RunPost(Common.START_RUN_URL + `/scheduler/authenticateOAuth?id_token=${fbToken}`, {
        id_token: fbToken,
        environmentID: workspaceID,
        oAuthInfo: {
            clientID: oauthClientID,
            clientSecret: oauthClientSecret,
            authURL: authURL,
            tokenURL: tokenURL,
            authCode: token,
            redirect_uri,
        }
    });
};
exports.SendOAuthAuthenticationRequestToScheduler = SendOAuthAuthenticationRequestToScheduler;
const IsValidEnvironmentConnection = (environment) => {
    let valid = true;
    const connection = environment.connectionAccount;
    if (connection === undefined || connection.trim() === "")
        valid = false;
    return valid;
};
exports.IsValidEnvironmentConnection = IsValidEnvironmentConnection;
const DoesConnectionTypeMatch = (connection, connectionTypeToCompare) => {
    return connection === connectionTypeToCompare;
};
const IsConnectionTypeOAuth = (connection) => {
    return DoesConnectionTypeMatch(connection.connectionType, EUserConnectionTypes.oauth);
};
exports.IsConnectionTypeOAuth = IsConnectionTypeOAuth;
const IsConnectionTypeBasic = (connection) => {
    return DoesConnectionTypeMatch(connection.connectionType, EUserConnectionTypes.basic);
};
exports.IsConnectionTypeBasic = IsConnectionTypeBasic;
/**
 * Helper function that checks if the fields oauthClientID & oauthClientSecret are "valid"
 * for a given environment
 * @param environment
 * @returns boolean
 */
const AreOAuthCredentialsValid = (environment) => {
    let valid = true;
    const { oauthClientID, oauthClientSecret } = environment;
    if (!oauthClientID || oauthClientID.trim() === "")
        valid = false;
    if (!oauthClientSecret || oauthClientSecret === "")
        valid = false;
    return valid;
};
exports.AreOAuthCredentialsValid = AreOAuthCredentialsValid;
/**
 * Determines that a user password is not an empty "" for an basic auth connection
 * @param credentials
 * @returns
 */
const IsBasicAuthPasswordValid = (credentials) => {
    let valid = true;
    if (!credentials)
        valid = false;
    if (credentials && (!credentials.snowflakePassword || credentials.snowflakePassword.trim() === ""))
        valid = false;
    return valid;
};
exports.IsBasicAuthPasswordValid = IsBasicAuthPasswordValid;
const IsValidIUserConnectionWithUser = (connection) => {
    let valid = true;
    if (!(0, exports.IsValidIUserConnection)(connection))
        return false;
    //valid but missing user
    if (!connection.connectionDetails.user || connection.connectionDetails.user.trim() === "")
        valid = false;
    return valid;
};
exports.IsValidIUserConnectionWithUser = IsValidIUserConnectionWithUser;
//# sourceMappingURL=ConnectionOperations.js.map