"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.listenMappingChangeCb = exports.getMappingCollectionRefAdmin = exports.getMappingCollectionRef = exports.GetSchemaFromV1MappingSafe = exports.GetDatabaseFromV1MappingSafe = exports.IsMappingsCollectionEmpty = exports.GetV1MappingsForNewOrg = exports.GetEmptyV1Mapping = exports.GetMappingsForYAML = exports.GetMappingsFromDevWorkspace = exports.GetDefaultStorageMappingFromV0Locations = exports.GetDefaultStorageMappingForCheckout = exports.DeleteEnvironmentFromMappings = exports.GetMappingUpdatesObjForNewDeployableEnvironment = exports.GetDefaultLocationForWorkspace = exports.GetEmptyDatabaseAndSchemaObject = exports.GetInitialMappingsInfoForNewOrg = exports.AddEmptyMappingsForNewDeployableEnvironment = exports.InitializeMappingsDataForNewDevWorkspace = exports.AddOrUpdateDevWorkspaceMappings = exports.GetFBV0MappingsCollection = exports.GetPhysicalLocationsFromMappings = exports.UpgradeWorkspaceMappingsV0ToV1ForDeployableEnv = exports.UpgradeDevWorkspaceMappingsV0ToV1 = exports.AreV0Locations = exports.AreMappingsV1 = exports.AreMappingsV0 = exports.MappingValidators = exports.RV0Mappings = exports.RV1Mappings = exports.RV1Mapping = exports.RPhysicalLocations = exports.RPhysicalLocation = exports.RV0Locations = exports.RV0Location = void 0;
const YAML = __importStar(require("yaml"));
const Logging = __importStar(require("./Logging"));
const CommonOperations = __importStar(require("./CommonOperations"));
const Workspaces = __importStar(require("./Workspaces"));
const Common = __importStar(require("./Common"));
const immer = __importStar(require("immer"));
const State = __importStar(require("./State"));
const R = __importStar(require("runtypes"));
const MappingLogger = Logging.GetLogger(Logging.LoggingArea.StorageLocations);
const RV0LocationID = R.String;
exports.RV0Location = R.Record({
    default: R.Boolean.optional(),
    locationName: R.String,
    id: R.Number.optional() // optional because old locations will have it, but new ones will not
});
exports.RV0Locations = R.Dictionary(exports.RV0Location, RV0LocationID);
////// PHYSICAL LOCATIONS
/**
 * The physical location interface has been repurposed.
 *
 * Originally, IPhysicalLocation was used to describe the mapping
 * definitions that were stored on an environment/workspace. That
 * interface was maintained under TV0Mappings.
 *
 * The physical location types below are more for utility purposes -
 * these aren't stored on metadata, they're just a type that we
 * use in our calculations. TPhysicalLocation is now the single interface
 * that we can convert either V0 or V1 mappings to.
 */
const RStorageLocationName = R.String;
exports.RPhysicalLocation = R.Record({
    locationName: R.String,
    database: R.String,
    schema: R.String
});
exports.RPhysicalLocations = R.Dictionary(exports.RPhysicalLocation, RStorageLocationName);
var EMappingVersions;
(function (EMappingVersions) {
    EMappingVersions[EMappingVersions["v0"] = 0] = "v0";
    EMappingVersions[EMappingVersions["v1"] = 1] = "v1";
})(EMappingVersions || (EMappingVersions = {}));
// V1
/**
 * This can be either workspace or environment ID.
 *
 * In state, we track the current dev workspace's mappings
 * definitions, as well as all deployable environments' mappings
 * definitions.
 *
 * The current dev workspace's mappings definitions are not
 * serialized into git. All environment mapping definitions
 * are.
 */
const REnvironmentID = R.String;
const RV1MappingDefinitionForEnvironment = R.Record({
    database: R.String.optional(),
    schema: R.String.optional()
});
const RV1MappingDefinitionDictionary = R.Dictionary(RV1MappingDefinitionForEnvironment, REnvironmentID);
exports.RV1Mapping = R.Record({
    mappingDefinitions: RV1MappingDefinitionDictionary
});
exports.RV1Mappings = R.Dictionary(exports.RV1Mapping, RStorageLocationName);
// V0
/**
 * V0 mapping definitions used to be known as physical locations.
 */
const RV0Mapping = R.Record({
    id: R.Number,
    locationName: R.String,
    database: R.String,
    schema: R.String
});
// TODO: verify this type
const RStorageLocationID = R.Union(R.Number, R.String);
exports.RV0Mappings = R.Dictionary(RV0Mapping, RStorageLocationID);
////// MAPPING VALIDATION/VERSION CHECKING
exports.MappingValidators = {
    [EMappingVersions.v0]: (mappings) => CommonOperations.ValidateRunTypeConstraint(exports.RV0Mappings, mappings),
    [EMappingVersions.v1]: (mappings) => CommonOperations.ValidateRunTypeConstraint(exports.RV1Mappings, mappings)
};
const AreMappingsV0 = (mappings) => {
    const validationResult = exports.MappingValidators[EMappingVersions.v0](mappings);
    return validationResult.success;
};
exports.AreMappingsV0 = AreMappingsV0;
const AreMappingsV1 = (mappings) => {
    const validationResult = exports.MappingValidators[EMappingVersions.v1](mappings);
    return validationResult.success;
};
exports.AreMappingsV1 = AreMappingsV1;
/**
 * Primarily used when grabbing locations from data.yaml.
 *
 * In v0, storage locations were serialized in git under locations,
 * and their definitions were stored under environments as what we now
 * know as TV0Mappings.
 *
 * In v1, mapping definitions are stored in git all in one place,
 * under .locations.
 *
 * @param mappingsFromGit ".locations" from data.yaml
 */
const AreV0Locations = (locationsFromGit) => {
    const validationResult = CommonOperations.ValidateRunTypeConstraint(exports.RV0Locations, locationsFromGit);
    return validationResult.success;
};
exports.AreV0Locations = AreV0Locations;
/**
 * Function that can take either an old collection of
 * storage locations or an old set of mappings from a workspace
 * and produce v1 mappings.
 *
 * Currently used when we read all workspace data from git, firestore,
 * or during the initial upgrade.
 *
 * @param mappings mappings object from the old FB collection
 * @param workspaces workspaces data in their pre-upgrade (v0) condition
 * @returns object of new mappingsV1
 */
const UpgradeDevWorkspaceMappingsV0ToV1 = (locationsV0, workspaces) => {
    const newMappingsObj = {};
    Object.keys(locationsV0).forEach(locationID => {
        const locationName = locationsV0[locationID].locationName;
        if (!newMappingsObj[locationName]) {
            newMappingsObj[locationName] = (0, exports.GetEmptyV1Mapping)();
        }
        Object.keys(workspaces).forEach(workspaceID => {
            const workspaceData = workspaces[workspaceID];
            const workspaceMappingData = Common.getValueSafe(workspaceData, ["mappings", locationID], null);
            if (workspaceMappingData) {
                newMappingsObj[locationName].mappingDefinitions[workspaceID] = {
                    database: workspaceMappingData.database,
                    schema: workspaceMappingData.schema
                };
            }
            else {
                newMappingsObj[locationName].mappingDefinitions[workspaceID] = (0, exports.GetEmptyDatabaseAndSchemaObject)();
            }
        });
    });
    return newMappingsObj;
};
exports.UpgradeDevWorkspaceMappingsV0ToV1 = UpgradeDevWorkspaceMappingsV0ToV1;
/**
 * If a deployable env has been deployed to, it needs a mappingsV1 property in V1 that reflects the state of the mappings as of the most recent deploy.
 * This is necessary for refreshes to keep working without the user having to re-deploy to manually create this change
 *
 * @param oldMappings mappings object from old FB collection
 * @param deployedWorkspaces the deployedWorkspaces object from the environment being upgraded (won't exist if the env was never deployed to)
 * @returns object of new mappingsV1 (note this does not have to and probably won't match the mappingsV1 that the dev workspace gets on upgrade)
 */
const UpgradeWorkspaceMappingsV0ToV1ForDeployableEnv = (deployedWorkspaces) => {
    // construct an object that represents what the mappings object would have been at the time of last deploy
    const oldMappings = {};
    // work through each deployedWorkspace in case a mapping is missing from one for any reason
    Object.values(deployedWorkspaces).forEach(workspaceData => {
        Object.keys(workspaceData.mappings).forEach(locationID => {
            if (!oldMappings[locationID]) {
                oldMappings[locationID] = workspaceData.mappings[locationID];
            }
        });
    });
    return (0, exports.UpgradeDevWorkspaceMappingsV0ToV1)(oldMappings, deployedWorkspaces);
};
exports.UpgradeWorkspaceMappingsV0ToV1ForDeployableEnv = UpgradeWorkspaceMappingsV0ToV1ForDeployableEnv;
/**
 * Takes in mappings, determines version and converts the info to TPhysicalLocation for compatibility with existing functions
 * @param mappings mappings object from redux
 * @param workspaceID the ID of the workspace to make the physical locations for
 * @returns TPhysicalLocations
 */
const GetPhysicalLocationsFromMappings = (mappings, workspaceID) => {
    if ((0, exports.AreMappingsV0)(mappings)) {
        return GetPhysicalLocationsFromV0Mappings(mappings);
    }
    else {
        const V1MappingValidationResult = exports.MappingValidators[EMappingVersions.v1](mappings);
        Common.assert(Logging.LoggingArea.StorageLocations, V1MappingValidationResult.success, `Invalid mappings version (GetPhysicalLocationsFromMappings). Error: ${V1MappingValidationResult.errorString}`);
        return GetPhysicalLocationsFromV1Mappings(mappings, workspaceID);
    }
};
exports.GetPhysicalLocationsFromMappings = GetPhysicalLocationsFromMappings;
const GetPhysicalLocationsFromV1Mappings = (mappings, workspaceID) => {
    const physicalLocations = {};
    Object.keys(mappings).forEach(locationName => {
        const database = (0, exports.GetDatabaseFromV1MappingSafe)(mappings, locationName, workspaceID);
        const schema = (0, exports.GetSchemaFromV1MappingSafe)(mappings, locationName, workspaceID);
        physicalLocations[locationName] = {
            database,
            schema,
            locationName
        };
    });
    return physicalLocations;
};
const GetPhysicalLocationsFromV0Mappings = (mappings) => {
    const physicalLocations = {};
    Object.values((mappings)).forEach(location => {
        physicalLocations[location.locationName] = location;
    });
    return physicalLocations;
};
const GetFBV0MappingsCollection = (orgID) => {
    const mappingsCollRef = (0, exports.getMappingCollectionRef)(orgID);
    return mappingsCollRef.get()
        .then(mappingsQuery => {
        const mappingsDocs = mappingsQuery.docs;
        if (!mappingsDocs.length) {
            return null;
        }
        else {
            const mappings = {};
            mappingsQuery.docs.forEach(doc => {
                const mappingData = doc.data();
                mappings[mappingData.id] = mappingData;
            });
            return mappings;
        }
    });
};
exports.GetFBV0MappingsCollection = GetFBV0MappingsCollection;
/**
 * Updates mappings object based on updates passed in. This does not update the Redux state, only creates a new mapping object to be serialized.
 * Can be used to update only one database or schema name, or can replace entire mappings object, and everything between.
 * @param workspaces all current workspaces
 * @param mappings all current mappings
 * @param mappingUpdates just the mappings currently being updated but adhering to mapping definition interface
 * @param targetWorkspaceIDForUpdate the id of the dev workspace being edited
 * @param addedEnvironmentID the id of a new environment to add to the current workspace's mappings
 * @returns the new mappings object
 */
const AddOrUpdateDevWorkspaceMappings = (workspaces, mappings, mappingUpdates, targetWorkspaceIDForUpdate, addedEnvironmentID) => {
    const workspaceIDsToUpdate = Object.keys(workspaces).filter(workspaceID => {
        return Workspaces.IsWorkspaceCurrentOrDeployable(workspaces[workspaceID], targetWorkspaceIDForUpdate);
    });
    /*
     * when a new non-dev workspace is added, the mappings are then updated before the workspaces object changes, so
     * we make sure it gets updated here
     */
    if (addedEnvironmentID && !(new Set(workspaceIDsToUpdate).has(addedEnvironmentID))) {
        workspaceIDsToUpdate.push(addedEnvironmentID);
    }
    const newMappings = immer.produce(mappings, draftState => {
        Object.keys(mappingUpdates).forEach(locationName => {
            // if the locationName is new, fill with empty database and schema names before overwriting with any updates
            if (!draftState[locationName]) {
                draftState[locationName] = (0, exports.GetEmptyV1Mapping)();
                workspaceIDsToUpdate.forEach(workspaceID => {
                    draftState[locationName].mappingDefinitions[workspaceID] = Object.assign(Object.assign({}, (0, exports.GetEmptyDatabaseAndSchemaObject)()), mappingUpdates[locationName].mappingDefinitions[workspaceID]);
                });
            }
            else {
                workspaceIDsToUpdate.forEach(workspaceID => {
                    const mappingDefinitionsForLocation = draftState[locationName].mappingDefinitions;
                    if (mappingUpdates[locationName].mappingDefinitions[workspaceID]) {
                        mappingDefinitionsForLocation[workspaceID] = Object.assign(Object.assign({}, mappingDefinitionsForLocation[workspaceID]), mappingUpdates[locationName].mappingDefinitions[workspaceID]);
                    }
                });
            }
        });
    });
    return newMappings;
};
exports.AddOrUpdateDevWorkspaceMappings = AddOrUpdateDevWorkspaceMappings;
/**
 * Creates mappingsV1 with blank database and schema info but then adds in any existing mappings passed in (in the case of duplicate)
 * @param newWorkspaceID ID of newly created workspace, not current workspace
 * @param workspaces all workspaces
 * @param mappings any existing mappings; will be empty in the case of a new empty dev workspace, but will have data if duplicating
 */
const InitializeMappingsDataForNewDevWorkspace = (newWorkspaceID, workspaces, mappings) => {
    // create mapping that dictionary that tracks correct workspace and environments
    const mappingDefinitionDictionary = {};
    // add key for current dev workspace
    mappingDefinitionDictionary[newWorkspaceID] = (0, exports.GetEmptyDatabaseAndSchemaObject)();
    // add keys for deployable environments
    Object.keys(workspaces).forEach(workspaceID => {
        const workspace = workspaces[workspaceID];
        if (!workspace.devEnv) {
            mappingDefinitionDictionary[workspaceID] = (0, exports.GetEmptyDatabaseAndSchemaObject)();
        }
    });
    // using above mapping dictionary as template, spread passed mapping definitions to format final TV1Mappings
    const newMappings = {};
    Object.keys(mappings).forEach(locationName => {
        newMappings[locationName] = (0, exports.GetEmptyV1Mapping)();
        newMappings[locationName].mappingDefinitions = Object.assign(Object.assign({}, mappingDefinitionDictionary), mappings[locationName]);
    });
    return newMappings;
};
exports.InitializeMappingsDataForNewDevWorkspace = InitializeMappingsDataForNewDevWorkspace;
const AddEmptyMappingsForNewDeployableEnvironment = (mappings, newWorkspaceID) => {
    const newMappings = immer.produce(mappings, draftState => {
        Object.keys(draftState).forEach(locationName => {
            draftState[locationName][newWorkspaceID] = (0, exports.GetEmptyDatabaseAndSchemaObject)();
        });
    });
    return newMappings;
};
exports.AddEmptyMappingsForNewDeployableEnvironment = AddEmptyMappingsForNewDeployableEnvironment;
/**
 * Creates the mappings object and defaultStorageMapping for a brand new org where there will be only the initial workspace and no mappings
 * @param workspaceID new workspace ID being created. For now this will always be 1, but this is resilient to changes to that in the future
 * @returns an object containing new mappings object and defaultStorageMapping string
 */
const GetInitialMappingsInfoForNewOrg = () => {
    const newMappings = (0, exports.GetV1MappingsForNewOrg)();
    const defaultStorageMapping = "SRC";
    return { newMappings, defaultStorageMapping };
};
exports.GetInitialMappingsInfoForNewOrg = GetInitialMappingsInfoForNewOrg;
const GetEmptyDatabaseAndSchemaObject = () => {
    return {
        database: "",
        schema: ""
    };
};
exports.GetEmptyDatabaseAndSchemaObject = GetEmptyDatabaseAndSchemaObject;
const GetDefaultLocationForWorkspace = (workspace) => {
    return Common.getValueSafe(workspace, ["defaultStorageMapping"], null);
};
exports.GetDefaultLocationForWorkspace = GetDefaultLocationForWorkspace;
const GetMappingUpdatesObjForNewDeployableEnvironment = (existingMappings, mappingUpdates, newEnvironmentID) => {
    const mappingsWithNewWorkspace = (0, exports.AddEmptyMappingsForNewDeployableEnvironment)(existingMappings, newEnvironmentID);
    let allMappingsUpdates;
    if (!mappingUpdates) {
        allMappingsUpdates = mappingsWithNewWorkspace;
    }
    else {
        // this ensures all storage loc/workspace combos are accounted for, while any updates from envDetails also get added
        allMappingsUpdates = immer.produce(mappingsWithNewWorkspace, draftState => {
            Object.keys(draftState).forEach(locationName => {
                /*
                 * relevant mappings on the updates object will have the scratch ID (currently 0), but the end result needs to have the
                 * mappings associated with the newly created ID (which is also where the empty mappings will be on mappingsWithNewWorkspace)
                 */
                if (mappingUpdates[locationName][Workspaces.ScratchWorkspaceID]) {
                    draftState[locationName][newEnvironmentID] = Object.assign(Object.assign({}, draftState[locationName][newEnvironmentID]), mappingUpdates[locationName][Workspaces.ScratchWorkspaceID]);
                }
            });
        });
    }
    return allMappingsUpdates;
};
exports.GetMappingUpdatesObjForNewDeployableEnvironment = GetMappingUpdatesObjForNewDeployableEnvironment;
const DeleteEnvironmentFromMappings = (currentMappings, environmentID) => {
    const newMappings = immer.produce(currentMappings, draftState => {
        Object.keys(draftState).forEach(locationName => {
            delete draftState[locationName].mappingDefinitions[environmentID];
        });
    });
    return newMappings;
};
exports.DeleteEnvironmentFromMappings = DeleteEnvironmentFromMappings;
/**
 * Will return existing default mapping if it exists on the data, and will figure out if a new default should
 * be chosen automatically (if storage locations exist without a default) otherwise
 * @param workspaceData data of interface AllWorkspaceData (from git specifically in the case of checkout)
 * @returns the existing defaultStorageMapping if exists, determines what to fill in if none exists
 */
const GetDefaultStorageMappingForCheckout = (workspaceData) => {
    if (workspaceData.defaultStorageMapping) {
        return workspaceData.defaultStorageMapping;
    }
    else if (workspaceData.locations && Object.keys(workspaceData.locations).length) {
        // if storage locations exist without a default already set, set the first one to default
        return Object.keys(workspaceData.locations)[0];
    }
    else {
        // set to null if no locations exist
        return null;
    }
};
exports.GetDefaultStorageMappingForCheckout = GetDefaultStorageMappingForCheckout;
const GetDefaultStorageMappingFromV0Locations = (locations) => {
    // .find ensures only the first one labeled default will be stored as default after upgrade if there's multiple; see CD-3050
    const defaultMappingID = Object.keys(locations).find(mappingID => locations[mappingID].default);
    return Common.getValueSafe(locations, [defaultMappingID, "locationName"], null);
};
exports.GetDefaultStorageMappingFromV0Locations = GetDefaultStorageMappingFromV0Locations;
const GetMappingsFromDevWorkspace = (allWorkspaces, devWorkspaceID) => {
    const mappingsV1 = Common.getValueSafe(allWorkspaces[devWorkspaceID], ["mappingsV1"], null);
    if (!mappingsV1) {
        return {};
    }
    else {
        return YAML.parse(mappingsV1);
    }
};
exports.GetMappingsFromDevWorkspace = GetMappingsFromDevWorkspace;
/**
 *
 * @param mappings all mappings (state.mappings)
 * @param currentWorkspaceID
 * @returns mappings, with the currentWorkspace mappings removed. we don't serialize
 * the current dev workspace mappings into git
 */
const GetMappingsForYAML = (mappings, currentWorkspaceID) => {
    const returnMappings = Common.CreateDeepClone(mappings);
    Object.values(returnMappings).forEach(mappingEntry => {
        delete mappingEntry.mappingDefinitions[currentWorkspaceID];
    });
    return returnMappings;
};
exports.GetMappingsForYAML = GetMappingsForYAML;
const GetEmptyV1Mapping = () => {
    return {
        mappingDefinitions: {}
    };
};
exports.GetEmptyV1Mapping = GetEmptyV1Mapping;
/**
 * @returns mappings to be used for the initialization of a brand
 * new org and workspace. if no mappings exist and this is the first
 * workspace being created, they always start with an SRC storage
 * location.
 */
const GetV1MappingsForNewOrg = () => {
    const newMappings = {
        SRC: {
            mappingDefinitions: {
                1: (0, exports.GetEmptyDatabaseAndSchemaObject)()
            }
        }
    };
    return newMappings;
};
exports.GetV1MappingsForNewOrg = GetV1MappingsForNewOrg;
/**
 * Helper function to determine if the mappings collection has any values, when empty return true (works with any mapping version)
 * @param mappings
 * @returns
 */
const IsMappingsCollectionEmpty = (mappings) => {
    const length = Object.keys(mappings).length;
    return !!!length;
};
exports.IsMappingsCollectionEmpty = IsMappingsCollectionEmpty;
const GetDatabaseFromV1MappingSafe = (mappings, storageLocationName, workspaceID) => {
    return Common.getValueSafe(mappings, [storageLocationName, "mappingDefinitions", workspaceID, "database"], "");
};
exports.GetDatabaseFromV1MappingSafe = GetDatabaseFromV1MappingSafe;
const GetSchemaFromV1MappingSafe = (mappings, storageLocationName, workspaceID) => {
    return Common.getValueSafe(mappings, [storageLocationName, "mappingDefinitions", workspaceID, "schema"], "");
};
exports.GetSchemaFromV1MappingSafe = GetSchemaFromV1MappingSafe;
const getMappingCollectionRef = (teamID) => {
    return CommonOperations.getOrgConfigDocRef(teamID).collection("mappings");
};
exports.getMappingCollectionRef = getMappingCollectionRef;
const getMappingCollectionRefAdmin = (teamID, firestore) => {
    return CommonOperations.getOrgConfigDocRefAdmin(firestore, teamID).collection("mappings");
};
exports.getMappingCollectionRefAdmin = getMappingCollectionRefAdmin;
const listenMappingChangeCb = (change, dispatch) => {
    const { type, doc } = change;
    if (type === "clear") {
        dispatch({ type: State.getClearAllEntityActionString(CommonOperations.ECollectionRefs.mappings) });
    }
    else if (type === "added") {
        dispatch(State.getCreateMapping(doc.data()));
    }
    else if (type === "modified") { //need state/action
        dispatch(State.getModifyMapping(doc.data()));
    }
    else if (type === "removed") {
        dispatch(State.getDeleteMapping(doc.id));
    }
    else {
        console.log("unknown update type", type);
        console.assert(false);
    }
};
exports.listenMappingChangeCb = listenMappingChangeCb;
//# sourceMappingURL=MappingOperations.js.map