"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AllScanners = exports.EProblemType = exports.EAllScanners = exports.ScannerStateSelectors = exports.EProblemAreas = exports.EProblemAreaKeys = exports.EModalTabKeys = exports.ERequiredSnowflakeUserConfig = exports.ProblemLogger = void 0;
const State = __importStar(require("./State"));
const StepTypes = __importStar(require("./StepTypes"));
const Common = __importStar(require("./Common"));
const Logging = __importStar(require("./Logging"));
const GitOperations = __importStar(require("./GitOperation"));
const ConnectionOperations = __importStar(require("./ConnectionOperations"));
const MappingOperations = __importStar(require("./MappingOperations"));
const StepOperations = __importStar(require("./StepOperations"));
exports.ProblemLogger = Logging.GetLogger(Logging.LoggingArea.Problems);
var ERequiredSnowflakeUserConfig;
(function (ERequiredSnowflakeUserConfig) {
    ERequiredSnowflakeUserConfig["configType"] = "Snowflake Settings";
    ERequiredSnowflakeUserConfig["snowflakeAccount"] = "Account";
    ERequiredSnowflakeUserConfig["snowflakePassword"] = "Password";
    ERequiredSnowflakeUserConfig["snowflakeUsername"] = "Username";
})(ERequiredSnowflakeUserConfig = exports.ERequiredSnowflakeUserConfig || (exports.ERequiredSnowflakeUserConfig = {}));
var EModalTabKeys;
(function (EModalTabKeys) {
    EModalTabKeys["environmentSettings"] = "Environment Settings";
    EModalTabKeys["storageMappings"] = "Storage Mappings";
    EModalTabKeys["userCredentials"] = "User Credentials";
})(EModalTabKeys = exports.EModalTabKeys || (exports.EModalTabKeys = {}));
var EProblemAreaKeys;
(function (EProblemAreaKeys) {
    EProblemAreaKeys["buildSettings"] = "buildSettings";
    EProblemAreaKeys["step"] = "step";
})(EProblemAreaKeys = exports.EProblemAreaKeys || (exports.EProblemAreaKeys = {}));
var EProblemAreas;
(function (EProblemAreas) {
    EProblemAreas["buildSettings"] = "Build Settings";
    EProblemAreas["userSettings"] = "User Settings";
    EProblemAreas["storageLocations"] = "Storage Locations";
    EProblemAreas["environments"] = "Environments";
    EProblemAreas["storageAndEnvironments"] = "Storage, Environments, and Current Workspace";
    EProblemAreas["environmentSettings"] = "Settings";
    EProblemAreas["graph"] = "Graph";
    EProblemAreas["nodeTypes"] = "Node Types";
})(EProblemAreas = exports.EProblemAreas || (exports.EProblemAreas = {}));
exports.ScannerStateSelectors = {
    stepTypesParsed: ["stepTypesParsed"],
    mappings: ["mappings"],
    workspaces: ["workspaces", "workspaces"],
    userConfig: ["userConfig"]
};
var EAllScanners;
(function (EAllScanners) {
    EAllScanners["MissingUserConfig"] = "MissingUserConfig";
    EAllScanners["StorageLocationProblems"] = "StorageLocationProblems";
    EAllScanners["DeletedStorageLocationsInUseProblems"] = "DeletedStorageLocationsInUseProblems";
    EAllScanners["CustomNodeTypeProblems"] = "CustomNodeTypeProblems";
})(EAllScanners = exports.EAllScanners || (exports.EAllScanners = {}));
var EProblemType;
(function (EProblemType) {
    EProblemType["immutableFolder"] = "immutableFolder";
    EProblemType["issue"] = "issue";
    EProblemType["warning"] = "warning";
})(EProblemType = exports.EProblemType || (exports.EProblemType = {}));
exports.AllScanners = {
    MissingConnectionsConfig: {
        dependencies: (currentState) => [currentState.auth, Object.values(currentState.userConnections), currentState.deploymentCredentials, currentState.workspaces],
        // only checks dev environment
        scan: (currentState) => {
            const currentWorkspaceID = currentState.currentWorkspace;
            const currentWorkspaceUserConnections = currentState.userConnections[currentWorkspaceID];
            const devWorkspace = Object.values(currentState.workspaces.workspaces).find((workspace) => workspace.id === currentWorkspaceID);
            if (!devWorkspace) {
                return [];
            }
            const orgID = currentState.auth.fbTeamID;
            const userID = currentState.auth.fbUserID;
            const { oauthClientID, oauthClientSecret, id, name, connectionAccount } = devWorkspace;
            Common.assertWithContext(Logging.LoggingArea.Problems, !!devWorkspace, Logging.frontEndGlobalContext, `devWorkspace does not exist. devWorkspace.id: ${devWorkspace.id}`);
            const connectionConfigProblems = [];
            const problemPayload = {
                hierarchy: [EProblemAreas.buildSettings, EProblemAreas.storageAndEnvironments, name],
                error: "",
                id: 0,
                type: EProblemType.warning,
                auxData: { workspaceData: { workspaceID: devWorkspace.id } },
            };
            // mostly 1stUX scenario - user added to userConnections once username/password fields are filled out and submitted
            if (!currentWorkspaceUserConnections) {
                if (!connectionAccount || connectionAccount.trim() === "") { // 
                    problemPayload.error = `Add a 'Connection' in the ${EProblemAreas.environmentSettings} tab`;
                }
                else {
                    problemPayload.error = `Add a Username and Password in the ${EModalTabKeys.userCredentials} tab`;
                    problemPayload.auxData.workspaceData.envTabKey = EModalTabKeys.userCredentials;
                }
                /**
                 * heirarchy refers to ag-grid tree format where you manually define the tree heirarchy by hand
                 * Pushing the key into the heirarchy tells the tree that this is a new level, please display this
                 * at that nested level,  0 is a placeholder that has no real significance at this time besides
                 * just making a new level in the heirarchy
                 */
                problemPayload.hierarchy.push("0");
                connectionConfigProblems.push(problemPayload);
                // if initial setup has occurred but no connection has been added
            }
            else if (!connectionAccount || connectionAccount.trim() === "") {
                const payloadUpdates = {
                    hierarchy: [EProblemAreas.buildSettings, EProblemAreas.storageAndEnvironments, "Setup", id],
                    error: `Add a 'Connection' in the ${EProblemAreas.environmentSettings} tab`,
                    id,
                };
                // problemPayload.hierarchy.push(id)
                connectionConfigProblems.push(Object.assign(Object.assign({}, problemPayload), payloadUpdates));
                // if initial setup is complete, connection is added, but missing credentials
            }
            else {
                const currentUserID = currentState.auth.fbUserID;
                const currentUserConnection = currentWorkspaceUserConnections[currentUserID];
                //setup inital connnection, but at least one other user in the org exists
                if (!currentUserConnection) {
                    if (!ConnectionOperations.IsValidEnvironmentConnection(devWorkspace)) { // 
                        problemPayload.error = `Add a 'Connection' in the ${EProblemAreas.environmentSettings} tab`;
                    }
                    else {
                        problemPayload.error = `Add a Username and Password in the ${EModalTabKeys.userCredentials} tab`;
                        problemPayload.auxData.workspaceData.envTabKey = EModalTabKeys.userCredentials;
                    }
                    problemPayload.hierarchy.push("0");
                    connectionConfigProblems.push(problemPayload);
                }
                else if (ConnectionOperations.IsConnectionTypeOAuth(currentUserConnection)) {
                    // if oauth is enabled, check for oauth credentials
                    if (!ConnectionOperations.AreOAuthCredentialsValid(devWorkspace)) {
                        const payloadUpdates = {
                            error: `Add a 'Client ID' & 'Client Secret' in the ${EProblemAreas.environmentSettings} tab`,
                            id,
                            auxData: { workspaceData: { workspaceID: devWorkspace.id, envTabKey: EModalTabKeys.environmentSettings } },
                        };
                        problemPayload.hierarchy.push(id);
                        connectionConfigProblems.push(Object.assign(Object.assign({}, problemPayload), payloadUpdates));
                    }
                    // if connection type is 'basic', check basic credentials
                }
                else if (ConnectionOperations.IsConnectionTypeBasic(currentUserConnection)) {
                    const devDeploymentKey = Object.keys(currentState.deploymentCredentials).find(credKey => credKey === devWorkspace.id.toString());
                    const devDeploymentCredentials = currentState.deploymentCredentials[devDeploymentKey];
                    // we allow spaces in our passwords
                    if (!ConnectionOperations.IsValidIUserConnectionWithUser(currentUserConnection) || (!ConnectionOperations.IsBasicAuthPasswordValid(devDeploymentCredentials))) {
                        const payloadUpdates = {
                            error: `Add a 'Username' and 'Password' in the ${EModalTabKeys.userCredentials} tab`,
                            id,
                            auxData: { workspaceData: { workspaceID: devWorkspace.id, envTabKey: EModalTabKeys.userCredentials } },
                        };
                        problemPayload.hierarchy.push(id);
                        connectionConfigProblems.push(Object.assign(Object.assign({}, problemPayload), payloadUpdates));
                    }
                }
                else { //unknown connection type, create update, and log
                    Common.assertWithContext(Logging.LoggingArea.Problems, false, { envID: devWorkspace.id, orgID: orgID, userID: userID }, `Unknown connection type: ${currentUserConnection.connectionType}`);
                }
            }
            return connectionConfigProblems;
        },
        getUI: (problem) => {
            const errorString = problem.error;
            const tab = `${EProblemAreaKeys.buildSettings}-1`;
            const actions = [
                State.getSetCurrentTabAction(tab),
                State.getShowStepAction(1, EProblemAreaKeys.buildSettings),
            ];
            return { errorString, actions };
        },
    },
    MissingUserConfig: {
        dependencies: (currentState) => [currentState.userConfig],
        scan: (currentState) => {
            const currentUserConfig = currentState.userConfig;
            const requiredUserConfigFields = [GitOperations.ERequiredGitUserConfig];
            const newProblems = [];
            const userConfigProblems = {};
            requiredUserConfigFields.forEach((userConfigArea, index) => {
                userConfigProblems[userConfigArea.configType] = [];
                Object.keys(requiredUserConfigFields[index]).forEach(configFieldKey => {
                    // configType is not a potential missing field, but rather the title of the area (ex. "Snowflake Settings")
                    if (configFieldKey === "configType")
                        return;
                    // may be undefined if field has not yet been focused and typed in the UI
                    if (!currentUserConfig[configFieldKey] || currentUserConfig[configFieldKey].trim() === "") {
                        userConfigProblems[userConfigArea.configType].push(userConfigArea[configFieldKey]);
                    }
                });
            });
            Object.keys(userConfigProblems).forEach(configTypeKey => {
                const hierarchy = ["User Settings", `${configTypeKey}`, 0];
                const missingFields = userConfigProblems[configTypeKey];
                if (!missingFields.length)
                    return;
                // creating numbered list of missing fields
                const missingFieldsString = missingFields.map((field, index) => `\n(${index + 1}) ${field}`).join();
                const errorString = `Please add: ${missingFieldsString}`;
                const problemPayload = {
                    hierarchy,
                    error: errorString,
                    id: 0,
                    type: EProblemType.issue
                };
                newProblems.push(problemPayload);
            });
            return newProblems;
        },
        getUI: (problem) => {
            const errorString = problem.error;
            const link = "/userSettings";
            return { errorString, link };
        },
    },
    StorageLocationProblems: {
        dependencies: (currentState) => [currentState.mappings, currentState.workspaces, currentState.currentWorkspace],
        scan: (currentState) => {
            const mappings = currentState.mappings;
            const workspaces = currentState.workspaces.workspaces;
            const currentWorkspaceID = currentState.currentWorkspace;
            const healthyWorkspaces = Object.values(workspaces).filter((workspace) => !!workspace.name && !!workspace.createdAt);
            // Mappings from state only contains mappings for current dev workspace and non-dev workspaces so ignore all other dev envs
            const healthyWorkspacesToUse = healthyWorkspaces.filter(workspace => workspace.id === currentWorkspaceID || !workspace.devEnv);
            const newProblems = () => {
                if (!mappings)
                    return [];
                const storageLocationProblems = [];
                healthyWorkspacesToUse.forEach(workspace => {
                    const { healthyStorageLocNames, unhealthyStorageLocs } = GetStorageLocsConfigStatusInfo(mappings, workspace.id);
                    healthyStorageLocNames.forEach(healthyLocName => {
                        // mappings in state may not map to mappings in env
                        if (!mappings[healthyLocName])
                            return;
                    });
                    unhealthyStorageLocs.forEach(storageLoc => {
                        if (!mappings[storageLoc.name])
                            return;
                        const workspaceName = workspace.name;
                        const problemPayload = {
                            hierarchy: [EProblemAreas.buildSettings, EProblemAreas.storageAndEnvironments, workspaceName, storageLoc.name],
                            error: storageLoc.errorString,
                            id: +storageLoc.name,
                            type: EProblemType.issue,
                            auxData: { workspaceData: { workspaceID: workspace.id, envTabKey: EModalTabKeys.storageMappings } },
                        };
                        storageLocationProblems.push(problemPayload);
                    });
                });
                return storageLocationProblems;
            };
            const checkInitialStorageSetup = () => {
                const hasMappings = Object.keys(mappings).length;
                const problemPayload = {
                    hierarchy: [EProblemAreas.buildSettings, EProblemAreas.storageAndEnvironments, "Setup", "0"],
                    error: "Create a new Storage Location before attempting to add Sources",
                    id: null,
                    type: EProblemType.warning,
                };
                if (!hasMappings)
                    return [problemPayload];
                return [];
            };
            return [...newProblems(), ...checkInitialStorageSetup()];
        },
        getUI: (problem) => {
            const errorString = problem.error;
            const tab = `${EProblemAreaKeys.buildSettings}-1`;
            const actions = [
                State.getSetCurrentTabAction(tab),
                State.getShowStepAction(1, EProblemAreaKeys.buildSettings),
            ];
            return { errorString, actions };
        },
    },
    DeletedStorageLocationsInUseProblems: {
        dependencies: (currentState) => [
            currentState.mappings,
            currentState.steps,
            currentState.stepTypesParsed
        ],
        scan: (currentState) => {
            const { mappings, steps, stepTypesParsed: stepTypesData } = currentState;
            const existingStorageLocationNames = Object.keys(mappings);
            const nodeTypeIDs = Object.keys(stepTypesData);
            const filteredSteps = StepOperations.GetStepsWithoutScratchStep(steps);
            const stepCounters = Object.keys(filteredSteps);
            const checkNodeTypeStorageLocations = () => {
                const problems = [];
                nodeTypeIDs.forEach(nodeTypeID => {
                    const defaultStorageLocation = Common.getValueSafe(stepTypesData[nodeTypeID].metadata, ["defaultStorageLocation"], null);
                    const nodeTypeName = Common.getValueSafe(stepTypesData[nodeTypeID], ["name"], null);
                    if (!existingStorageLocationNames.includes(defaultStorageLocation) && defaultStorageLocation !== null) { // break this check out to a global util? for use in both ts & tsx files
                        problems.push({
                            hierarchy: [
                                EProblemAreas.buildSettings,
                                "Node Types",
                                nodeTypeID
                            ],
                            error: `Default storage location for Node Type ${nodeTypeName} no longer exists. Please assign a new default storage location for this node type.`,
                            id: 0,
                            type: EProblemType.issue
                        });
                    }
                });
                return problems;
            };
            const checkNodeStorageLocations = () => {
                const problems = [];
                stepCounters.forEach(stepCounter => {
                    const locationName = Common.getValueSafe(steps[stepCounter].operation, ["locationName"], null);
                    const nodeName = Common.getValueSafe(steps[stepCounter].operation, ["name"], null);
                    if (!existingStorageLocationNames.includes(locationName)) {
                        problems.push({
                            hierarchy: [
                                EProblemAreas.graph,
                                "Nodes",
                                nodeName,
                                "Node Properties"
                            ],
                            error: `Storage location for Node ${nodeName} no longer exists. Please assign a new storage location for this node.`,
                            id: +stepCounter,
                            type: EProblemType.issue
                        });
                    }
                });
                return problems;
            };
            return [
                ...checkNodeTypeStorageLocations(),
                ...checkNodeStorageLocations()
            ];
        },
        getUI: (problem) => {
            const problemIsNode = problem.hierarchy[0] === EProblemAreas.graph;
            const tabID = problemIsNode ? problem.id : 1;
            const tabType = problemIsNode ? EProblemAreaKeys.step : EProblemAreaKeys.buildSettings;
            const actions = [
                State.getSetCurrentTabAction(`${tabType}-${tabID}`),
                State.getShowStepAction(tabID, tabType)
            ];
            return { errorString: problem.error, actions };
        }
    },
    CustomNodeTypeProblems: {
        dependencies: (currentState) => [currentState.stepTypesParsed],
        scan: (currentState) => {
            const stepTypesParsed = currentState.stepTypesParsed;
            const customStepTypesParsed = StepTypes.GetCustomStepTypes(stepTypesParsed);
            const customNodeTypeProblems = [];
            Object.keys(customStepTypesParsed).forEach(templateCounter => {
                const stepTypeMetadata = customStepTypesParsed[templateCounter].metadata;
                const nodeTypeIdentifier = customStepTypesParsed[templateCounter].name || `with ID ${templateCounter}`;
                const hierarchy = [EProblemAreas.buildSettings, "Node Types", nodeTypeIdentifier];
                if (!!stepTypeMetadata.error) {
                    const problemPayload = {
                        hierarchy,
                        error: stepTypeMetadata.error.errorString,
                        id: +templateCounter,
                        type: EProblemType.issue
                    };
                    customNodeTypeProblems.push(problemPayload);
                }
            });
            return customNodeTypeProblems;
        },
        getUI: (problem) => {
            const getErrorString = () => {
                const nodeName = problem.hierarchy[2];
                if (problem.error.trim() === "Error" || !problem.error || !problem.error.includes("{")) {
                    return `There are one or more errors with ${nodeName}. Last valid Node Definition will be used until errors resolved.`;
                }
                const nodeErrorList = problem.error.split("{")[1].split("}")[0].split(`",`);
                return `${nodeName}: ${nodeErrorList}`;
            };
            const errorString = getErrorString();
            const templateCounter = problem.id.toString();
            const modelTabKey = `templateEditor-${templateCounter}`;
            const actions = [
                State.getSetCurrentTabAction(modelTabKey),
                State.getShowStepAction(templateCounter, "templateEditor"),
                State.setEditorNodeTypeAction(templateCounter),
            ];
            return { errorString, actions };
        },
    },
    // will be implemented in next iteration
    // StepNodeProblems: {
    //     dependencies: (currentState)=> [ currentState.steps ],
    //     scan: (currentState): IScannerProblem[] => {
    //         const allSteps: Runner.IStepData[] = currentState.steps
    //         const stepNodeProblems: any = []
    //         // collects empty JOINS issues for single and multisource
    //         Object.keys(allSteps).forEach(stepKey => {
    //             const thisStepOperation = allSteps[stepKey].operation
    //             const stepCounter = +stepKey
    //             const stepSourceMapping = thisStepOperation.metadata.sourceMapping
    //             const stepName = thisStepOperation.name || `Node ${stepCounter}`
    //             const hierarchy = [ "Nodes", stepName, stepCounter ]          
    //             if (stepKey === "scratch" || thisStepOperation.type === Runner.EStepTypes.sourceInput) return
    //             const problemPayload = {
    //                 hierarchy,
    //                 error: "JOIN condition missing",
    //                 id: stepCounter,
    //                 type: EProblemType.warning
    //             }
    //             // if multisource
    //             if (thisStepOperation.isMultisource) {
    //                 stepSourceMapping.forEach(mapping => {                    
    //                     if (mapping.join.joinCondition.includes("/*COLUMN*/")) {
    //                         problemPayload.error = "JOIN condition missing qualified node name"                        
    //                     }                         
    //                     problemPayload.hierarchy = [ "Nodes", `${stepName} (multisource)`, mapping.name, stepCounter ]
    //                     stepNodeProblems.push(problemPayload)
    //                 })
    //             // singlesource
    //             } else {
    //                 if (!!stepSourceMapping) {
    //                     if (stepSourceMapping[0].join.joinCondition.trim() === "") {
    //                         stepNodeProblems.push(problemPayload)
    //                     }
    //                 } else {
    //                     if (thisStepOperation.metadata.join.joinCondition.trim() === "") {
    //                         stepNodeProblems.push(problemPayload)
    //                     }
    //                 }
    //             }
    //         })
    //         return stepNodeProblems
    //     },
    //     getUI: (problem: IScannerProblem) => {
    //         const errorString = problem.error
    //         const stepCounter = problem.id
    //         const modelTabKey = `step-${stepCounter}`
    //         const actions = [
    //             State.getShowStepAction(stepCounter, "step"),
    //             State.getSetCurrentTabAction(modelTabKey),
    //         ]
    //         return { errorString, actions }
    //     },
    // } as IScannerConfig
};
// HELPERS
/**
 * Returns an array of problematic storage location objects containing the location's ID as well as an errorString describing the issue
 * @param envMappings
 * @returns
 */
const GetProblemStorageLocsData = (envMappings, workspaceID) => {
    return Object.keys(envMappings).reduce((envProblems, locationName) => {
        const database = MappingOperations.GetDatabaseFromV1MappingSafe(envMappings, locationName, workspaceID);
        const schema = MappingOperations.GetSchemaFromV1MappingSafe(envMappings, locationName, workspaceID);
        let errorString = "";
        if (database.trim() === "" && schema.trim() === "") {
            errorString = `${locationName} is missing a valid Database and Schema`;
        }
        else if (schema.trim() === "") {
            errorString = `${locationName} is missing a valid Schema`;
        }
        else if (database.trim() === "") {
            errorString = `${locationName} is missing a valid Database`;
        }
        else
            return envProblems;
        const problemPayload = {
            name: locationName,
            errorString: `${errorString}, please update your Storage Mappings`
        };
        envProblems.push(problemPayload);
        return envProblems;
    }, []);
};
/**
 * Returns an array of storage location IDs for locations with both a filled out database and schema
 * @param envMappings
 * @param mappings
 * @returns
 */
const GetValidStorageLocNames = (envMappings, workspaceID) => {
    return Object.keys(envMappings).filter(locationName => {
        const database = MappingOperations.GetDatabaseFromV1MappingSafe(envMappings, locationName, workspaceID);
        const schema = MappingOperations.GetSchemaFromV1MappingSafe(envMappings, locationName, workspaceID);
        return (database.trim() !== "" && schema.trim() !== "");
    });
};
const GetStorageLocsConfigStatusInfo = (envMappings, workspaceID) => {
    let healthyStorageLocNames = [];
    let unhealthyStorageLocs = [];
    healthyStorageLocNames = GetValidStorageLocNames(envMappings, workspaceID);
    const invalidStorageLocs = GetProblemStorageLocsData(envMappings, workspaceID);
    unhealthyStorageLocs = [...invalidStorageLocs];
    return { healthyStorageLocNames, unhealthyStorageLocs };
};
//# sourceMappingURL=ProblemsScannerOperations.js.map