"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getYamlContentForStep = exports.getDataContent = exports.getDataYaml = exports.getYamlContent = exports.GetCollectionFromContent = exports.SerializeContent = exports.getReduxStateFromYaml = exports.ValidateCommitContent = exports.readme = void 0;
const immer = __importStar(require("immer"));
const YAML = __importStar(require("yaml"));
const Templates = __importStar(require("./Templates"));
const Immer = __importStar(require("immer"));
const Workspaces = __importStar(require("./Workspaces"));
const MappingOperations = __importStar(require("./MappingOperations"));
exports.readme = `# Welcome from Coalesce!
`;
const InvalidMarkers = ["refs/heads/", "<<<<<<<", "=======", ">>>>>>>"];
/**
 * Validates that the values coming in from a commit are in the correct
 * format. If they aren't, this throws an exception.
 *
 * @param commitText The yaml to parse and then validate
 * @param fileName The name of the yaml file being validated
 */
const ValidateCommitContent = (commitText, fileName) => {
    // don't parse yaml if we detect merge conflict markers
    InvalidMarkers.forEach(invalidMarker => {
        if (commitText.includes(invalidMarker)) {
            throw new Error("Merge conflict markers detected");
        }
    });
    const parsedYAML = YAML.parse(commitText);
    if (fileName === "data.yml") {
        //validate parent runType
        Templates.ValidateRunTypeWithThrow(Templates.AllDataYamlDefinitions, parsedYAML);
        //after success, then validate the stepTypes
        const parsedStepTypes = Immer.produce(parsedYAML.stepTypes, (draftState) => {
            Object.entries(parsedYAML.stepTypes).forEach(([key, stepType]) => {
                draftState[key].metadata.nodeMetadataSpec = YAML.parse(stepType.metadata.nodeMetadataSpec);
            });
        });
        //validate the parsed StepTypes dictionary
        Templates.ValidateRunTypeWithThrow(Templates.RStepTypesParsed, parsedStepTypes);
    }
    else if (fileName.includes("nodes/")) {
        //do nothing for now, create run type for nodes in next pass
    }
};
exports.ValidateCommitContent = ValidateCommitContent;
const getReduxStateFromYaml = (stepsYamlStrings, dataYamlString) => {
    const steps = {};
    stepsYamlStrings.forEach(stepYamlString => {
        try {
            //YAML parse will fail with invalid markers (<<<<<, >>>>>> from conflicts)
            const content = YAML.parse(stepYamlString);
            const stepsContent = content.steps;
            for (let key in stepsContent) {
                const { stepCounter, operation } = stepsContent[key];
                steps[stepCounter] = stepsContent[key];
                steps[stepCounter].name = operation.name;
            }
        }
        catch (error) {
            const splitNodeYaml = stepYamlString.split("\n");
            const possibleNodeName = splitNodeYaml[1].trim();
            throw new Error(`Failed to parse yaml for node ${possibleNodeName}. Description: ${error}`);
        }
    });
    let runResults = {};
    for (const key in steps) {
        runResults[key] = {};
        /*
         * TODO: how to handle serialization?
         * const iconfig = getConfigFromSQLString(steps[key].operation.sql)
         * tableRefs[key] = iconfig.tableRefs
         * TODO: how to handle failure?
         */
    }
    let dataContent = {
        environments: "",
        locations: "",
        subgraphs: "",
        jobs: "",
        macros: "",
        stepTypes: "",
        defaultStorageMapping: "",
        version: Workspaces.EWorkspaceVersions.v0,
        projects: "",
        packages: ""
    };
    if (dataYamlString) {
        dataContent = YAML.parse(dataYamlString);
    }
    const subgraphContent = dataContent.subgraphs || {};
    let folders = (0, exports.GetCollectionFromContent)(subgraphContent);
    const macroContent = dataContent.macros || {};
    let macros = (0, exports.GetCollectionFromContent)(macroContent);
    const packageContent = dataContent.packages || {};
    let packages = (0, exports.GetCollectionFromContent)(packageContent);
    const projectsContent = dataContent.projects || {};
    const projects = {};
    for (let key in projectsContent) {
        projects[key] = projectsContent[key];
    }
    Object.keys(folders).forEach(key => {
        folders[key].folderCounter = folders[key].id;
        delete folders[key].id;
    });
    const jobsContent = dataContent.jobs || {};
    let jobs = (0, exports.GetCollectionFromContent)(jobsContent);
    Object.keys(jobs).forEach(key => {
        jobs[key].jobCounter = jobs[key].id;
        delete jobs[key].id;
    });
    const stepTypesContent = dataContent.stepTypes || {};
    let stepTypes = (0, exports.GetCollectionFromContent)(stepTypesContent);
    const version = dataContent.version || Workspaces.EWorkspaceVersions.v0;
    const { mappings, defaultStorageMapping } = GeTV1MappingsAndDefaultStorageMappingFromDataContent(dataContent);
    return {
        runResults, steps, mappings, folders, jobs, macros, stepTypes, defaultStorageMapping, version, projects, packages
    };
};
exports.getReduxStateFromYaml = getReduxStateFromYaml;
const GeTV1MappingsAndDefaultStorageMappingFromDataContent = (dataContent) => {
    const environmentContent = dataContent.environments || {};
    let environments = (0, exports.GetCollectionFromContent)(environmentContent);
    const locationContent = dataContent.locations || {};
    let mappings = {};
    let defaultStorageMapping = null;
    if (Object.keys(locationContent).length) {
        let mappingsCollection;
        if (MappingOperations.AreV0Locations(locationContent)) {
            mappingsCollection = (0, exports.GetCollectionFromContent)(locationContent);
            mappings = MappingOperations.UpgradeDevWorkspaceMappingsV0ToV1(mappingsCollection, environments);
            defaultStorageMapping = MappingOperations.GetDefaultStorageMappingFromV0Locations(locationContent);
        }
        else {
            // after v0, ids not used in mappings, so GetCollectionFromContent would throw an error
            mappings = locationContent;
            defaultStorageMapping = dataContent.defaultStorageMapping || null;
        }
    }
    return { mappings, defaultStorageMapping };
};
const SerializeContent = (content, options) => {
    return YAML.stringify(content, options);
};
exports.SerializeContent = SerializeContent;
const GetCollectionFromContent = (content) => {
    const collection = {};
    for (let key in content) {
        const id = content[key].id;
        collection[id] = content[key];
    }
    return collection;
};
exports.GetCollectionFromContent = GetCollectionFromContent;
const getYamlContent = (steps, tasks, folders) => {
    let allYamlContent = {};
    let stepYamlContent = {};
    //clean data     
    const cleanStepCopy = immer.produce(steps, (draftState) => {
        Object.keys(steps).forEach(step => {
            delete draftState[step].x;
            delete draftState[step].y;
            delete draftState[step].inputs;
            delete draftState[step].outputs;
            delete draftState[step].name;
            draftState[step].stepCounter = step;
        });
    });
    //set data in yaml with correct Table Name
    Object.keys(cleanStepCopy).forEach(step => {
        const modelName = cleanStepCopy[step].operation.name;
        stepYamlContent[modelName + "-" + step] = cleanStepCopy[step];
    });
    allYamlContent["steps"] = stepYamlContent;
    allYamlContent = Object.assign(Object.assign({}, allYamlContent), (0, exports.getDataContent)(tasks, folders));
    return YAML.stringify(allYamlContent, { toStringDefaults: { blockQuote: "literal" }, sortMapEntries: true });
};
exports.getYamlContent = getYamlContent;
const getDataYaml = (locations, defaultStorageMapping, folders, jobs, macros, stepTypes, version, projects, packages) => {
    return YAML.stringify((0, exports.getDataContent)(locations, defaultStorageMapping, folders, jobs, macros, stepTypes, version, projects, packages), { toStringDefaults: { blockQuote: "literal" }, sortMapEntries: true });
};
exports.getDataYaml = getDataYaml;
const getDataContent = (locations, defaultStorageMapping, folders, jobs, macros, stepTypes, version, projects, packages) => {
    let allYamlContent = {};
    if (packages) {
        allYamlContent["packages"] = packages;
    }
    if (locations) {
        let locationYamlContent = {};
        Object.keys(locations).forEach(locationName => {
            locationYamlContent[locationName] = locations[locationName];
        });
        allYamlContent["locations"] = locationYamlContent;
    }
    // null is a valid value and should be added to the yaml
    if (defaultStorageMapping !== undefined) {
        allYamlContent["defaultStorageMapping"] = defaultStorageMapping;
    }
    if (folders) {
        let folderYamlContent = {};
        Object.keys(folders).forEach(folderCounter => {
            const folderName = `Subgraph-${folderCounter}`;
            folderYamlContent[folderName] = {};
            folderYamlContent[folderName].name = folders[folderCounter].name;
            folderYamlContent[folderName].steps = folders[folderCounter].steps;
            folderYamlContent[folderName].id = parseInt(folderCounter);
        });
        allYamlContent["subgraphs"] = folderYamlContent;
    }
    if (jobs) {
        let jobYamlContent = {};
        Object.keys(jobs).forEach(jobCounter => {
            const jobName = `Job-${jobCounter}`; //todo: enforce job & subgraph name uniqueness?
            jobYamlContent[jobName] = {};
            jobYamlContent[jobName].name = jobs[jobCounter].name;
            jobYamlContent[jobName].steps = jobs[jobCounter].steps;
            jobYamlContent[jobName].subgraphs = jobs[jobCounter].subgraphs;
            jobYamlContent[jobName].id = parseInt(jobCounter);
        });
        allYamlContent["jobs"] = jobYamlContent;
    }
    if (macros) {
        let macroYamlContent = {};
        Object.keys(macros).forEach(macroCounter => {
            const macroName = `Macro-${macroCounter}`;
            macroYamlContent[macroName] = {};
            macroYamlContent[macroName].macroString = macros[macroCounter].macroString;
            macroYamlContent[macroName].id = macroCounter;
            if (macros[macroCounter].packageInfo) {
                macroYamlContent[macroName]["packageInfo"] = macros[macroCounter].packageInfo;
            }
        });
        allYamlContent["macros"] = macroYamlContent;
    }
    if (stepTypes) {
        let stepTypeYamlContent = {};
        Object.keys(stepTypes).forEach(stepTypeID => {
            const stepTypeName = `StepType-${stepTypeID}`;
            stepTypeYamlContent[stepTypeName] = {};
            stepTypeYamlContent[stepTypeName].isDisabled = stepTypes[stepTypeID].isDisabled;
            stepTypeYamlContent[stepTypeName].version = stepTypes[stepTypeID].version;
            stepTypeYamlContent[stepTypeName].metadata = stepTypes[stepTypeID].metadata;
            stepTypeYamlContent[stepTypeName].name = stepTypes[stepTypeID].name;
            stepTypeYamlContent[stepTypeName].id = stepTypeID;
            if (stepTypes[stepTypeID].packageInfo) {
                stepTypeYamlContent[stepTypeName]["packageInfo"] = stepTypes[stepTypeID].packageInfo;
            }
        });
        allYamlContent["stepTypes"] = stepTypeYamlContent;
    }
    if (version) {
        allYamlContent["version"] = version;
    }
    if (projects) {
        let projectsYamlContent = {};
        Object.keys(projects).forEach(projectID => {
            projectsYamlContent[projectID] = {};
            projectsYamlContent[projectID].name = projects[projectID].name;
            if (projectsYamlContent[projectID].packageInfo) {
                projectsYamlContent[projectID]["packageInfo"] = projects[projectID].packageInfo;
            }
        });
        allYamlContent["projects"] = projectsYamlContent;
    }
    return allYamlContent;
};
exports.getDataContent = getDataContent;
const getYamlContentForStep = (step, stepCounter) => {
    let yamlContent = {};
    //clean data     
    const cleanStepCopy = immer.produce(step, (draftState) => {
        delete draftState.x;
        delete draftState.y;
        delete draftState.inputs;
        delete draftState.outputs;
        delete draftState.name;
        draftState.stepCounter = stepCounter;
    });
    const modelName = cleanStepCopy["operation"].name;
    yamlContent[modelName + "-" + stepCounter] = cleanStepCopy;
    return YAML.stringify({ steps: yamlContent }, { sortMapEntries: true });
    //set data in yaml with correct Table Name
};
exports.getYamlContentForStep = getYamlContentForStep;
//# sourceMappingURL=VersionControlActions.js.map